;******************************************************************
;*** FILENAME  : DEBUG.INC FOR PIC16F877 ONLY (20 MHz. HS MODE) 	 
;*** ASSEMBLER : MPASM V02.80 
;*** WRITTEN BY: Natthapol Wongsoonthornchai : 21 MAY 2002 
;*** COMPANY   : Innovative Experiment Co.,Ltd. Bangkok, Thailand
;*** RESOURCE  : RC6 pin, 3-level stack, 14-RAM in bank2 (address 0x120-0x12D)
;*** DETAIL    : This program use to debug your source program at run time (but not real-time). It Save these register before peerform W, STATUS, PCLATH and restore when finished. This program communicate with terminal program on PC such as Hyper Terminal via Serial RS-232 port at 9600 bps baud 8N1 format 	  
;******************************************************************
	  

	LIST	P=16F877,W=-302,W=-306  ; Suppress cross bank and page

;***************************_DEBUG_REGISTER_***********************

_PCL_	EQU	0x02	; Bank 0
_STATUS_	EQU	0x03
_RCSTA_	EQU	0x18
_TXREG_	EQU	0x19
_PCLATH_	EQU	0x0A
_PIR1_	EQU	0x0C

_TRISC_	EQU	0x87	; Bank 1 
_TXSTA_	EQU	0x98
_SPBRG_	EQU	0x99

_TXIF_	EQU	0x04	; Bit and flag
_SPEN_	EQU	0x07
_TXEN_	EQU	0x05
_BRGH_	EQU	0x02

     CBLOCK	0x120		; RAM in bank2 
      _CHAR0_,_CHAR1_,_CHAR2_,_CHAR3_,_CHAR4_,_CHAR5_,_DATA_OUT_,_COUNT8_      
        ENDC
         CBLOCK   0x128
          _SAVE_STATUS_,_SAVE_PCLATH_,_SAVE_W_,_DELAY0_,_DELAY1_,_DELAY2_
            ENDC
;***************** MACRO *************************************

DEBUG	MACRO	ASCII0,ASCII1,ASCII2,ASCII3,ASCII4,ASCII5,_REG_ADDR_,_REG_DELAY_
	BANKSEL	_SAVE_W_	 ; Select bank2
	MOVWF	_SAVE_W_	 ; Save W
	SWAPF	_PCLATH_,0      ; Save PCLATH
	MOVWF	_SAVE_PCLATH_	 ;
	SWAPF	_SAVE_PCLATH_,1 ;	
	PAGESEL	_SAVE_REGISTER_ ; Change PCLATH  
	CALL	_SAVE_REGISTER_ ; Save W,STATUS,PCLATH before debug		
	MOVLW	_REG_DELAY_	 ; Load delay time
	MOVWF	_DELAY2_
	MOVLW	ASCII0	 ; Send 6 ASCII
	MOVWF	_CHAR0_
	MOVLW	ASCII1
	MOVWF	_CHAR1_
	MOVLW	ASCII2
	MOVWF	_CHAR2_
	MOVLW	ASCII3
	MOVWF	_CHAR3_
	MOVLW	ASCII4
	MOVWF	_CHAR4_
	MOVLW	ASCII5
	MOVWF	_CHAR5_
	MOVFW	_REG_ADDR_
               MOVWF          _DATA_OUT_
	CALL	_DEBUGOUT_
	ENDM		   ; Emd macro

;**********************  TX & DELAY  ************************
	ORG	0x1F90
_SAVE_REGISTER_
	SWAPF	_STATUS_,0	   ; Save all flag except RP1,RP0
	MOVWF	_SAVE_STATUS_	
	RETURN
_DEBUGOUT_	
	BANKSEL	_TRISC_	   ; Select TRISC
	BSF	_TRISC_,6	   ; RC6 as input
	MOVLW	.129	   ; Baudrate = 9600 bps @Fosc 20 MHz.
	MOVWF	_SPBRG_	
	BSF	_TXSTA_,_BRGH_    ; High speed	
	BSF	_TXSTA_,_TXEN_    ; TX enable

	BANKSEL	_RCSTA_	   ; Bank 0
	BSF	_RCSTA_,_SPEN_    ; Serial port enable

	BANKSEL	_CHAR0_	   ; Bank2
	MOVFW	_CHAR0_
	CALL	_SEND_DATA_
	MOVFW	_CHAR1_
	CALL	_SEND_DATA_
	MOVFW	_CHAR2_
	CALL	_SEND_DATA_
	MOVFW	_CHAR3_
	CALL	_SEND_DATA_
	MOVFW	_CHAR4_
	CALL	_SEND_DATA_
	MOVFW	_CHAR5_
	CALL	_SEND_DATA_
	CALL	_SEND_EQUAL_
	MOVLW	'$'	     ; Hexadecimal prefix
	CALL	_SEND_DATA_
	MOVFW	_DATA_OUT_       ; Debug register
	CALL	_SWAP_FILE_    
	CALL	_SWAP_FILE_	   

	CALL	_SEND_SPACE_     ; Send ASCII = and blank space
	CALL	_SEND_EQUAL_
	MOVLW	'%'	    ; Binary prefix
	CALL	_SEND_DATA_
	MOVLW	.8	  ; Rotate 8 times for binary value
	MOVWF	_COUNT8_	
_ASCII4BIN_	RLF	_DATA_OUT_,1
	BC	_CARRY_SET_
_CARRY_CLEAR_
 	MOVLW	'0'
	GOTO	_SEND_0_
_CARRY_SET_	MOVLW	'1'
_SEND_0_	CALL	_SEND_DATA_
_CHECK8_	DECFSZ	_COUNT8_,1
	GOTO	_ASCII4BIN_

	MOVLW	0x0A	   ; Line feed
	CALL	_SEND_DATA_
	MOVLW	0x0D	   ; Carriage return
	CALL	_SEND_DATA_
	GOTO	_DEBUG_DELAY_     ;_return_	

_SWAP_FILE_	MOVLW	B'00011111'	   ; Set up for look-up table
	MOVWF	_PCLATH_	
	SWAPF	_DATA_OUT_,1
	MOVFW	_DATA_OUT_
	ANDLW	B'00001111'	   ; Mask bit
	CALL	_DEBUG_TABLE_
	CALL	_SEND_DATA_
	RETURN
_DEBUG_TABLE_
	ADDWF	_PCL_,1
	DT	"0123456789ABCDEF" 
_SEND_DATA_	
	BANKSEL	_PIR1_	    ; Bank 0 
	BTFSS	_PIR1_,_TXIF_      ; Skip if TXREG empty
	GOTO	$-1	    ; Not empty then wait 
	MOVWF	_TXREG_	    ; Send data to RS-232
	BANKSEL	_DATA_OUT_	    ; Return to bank2
	RETURN

_SEND_EQUAL_	MOVLW	'='
	CALL	_SEND_DATA_
_SEND_SPACE_	MOVLW	' '
	CALL	_SEND_DATA_
	RETURN
_DEBUG_DELAY_  
	TSTF	_DELAY2_ 
	BZ	_RETURN_	   ; Branch if not delay	
	CLRF	_DELAY0_
	CLRF	_DELAY1_
	DECFSZ	_DELAY0_,1
	GOTO	$-1
	DECFSZ	_DELAY1_,1
	GOTO	$-3
	DECFSZ	_DELAY2_,1
	GOTO	$-5
_RETURN_	
	MOVFW	_SAVE_PCLATH_     ; Restore PCLATH
	MOVWF	_PCLATH_		
	SWAPF	_SAVE_STATUS_,0   ; Restore STATUS
	MOVWF	_STATUS_
	SWAPF	_SAVE_W_,1	   ; Restore W
	SWAPF	_SAVE_W_,0	 
	BANKSEL	_STATUS_	   ; Return in bank0
	RETURN	
