;***********************************************************
; Filename :	lab0901.asm
; Descripton :	Demonstrate Timer0 operation in counter mode
; MCU :		PIC16F877A
; Copyright : 	2002-2003 Innovative Experiment
;************************************************************
	LIST	P=16F877,W=-302
	__CONFIG	0x3D7A	; 20 MHz. HIGH SPEED GAIN
	
TMR0	EQU	0x01	; Bank0
PORTB	EQU	0x06
INTCON	EQU	0x0B

OPTION_REG	EQU	0x81	; Bank1
TRISA	EQU	0x85
TRISB	EQU	0x86	

GIE	EQU	7
T0IE	EQU	5
T0IF	EQU	2
;************************************************************
	ORG	0x00
	GOTO	START
	
	ORG	0x04
	GOTO	INTERRUPT
START:	
	CALL	PORT_SETUP
	CALL	TMR0_SETUP
	CALL	INTERRUPT_SETUP
	CLRF	TMR0	; Clear TMR0
	BSF	INTCON,GIE	; Interrupt enable
	GOTO	$	; Wait for interrupt

INTERRUPT:	
	BCF	INTCON,T0IF	; Clear overflow flag
	RRF	PORTB,1	; Rotate right
	RETFIE	
	
;********************  SUBROUTINE  *************************	
PORT_SETUP	
	BANKSEL	TRISB	; Select TRISB <bank1>
	CLRF	TRISB	; Port B = output
	BSF	TRISA,4	; Make sure RA4 as input
	BANKSEL	PORTB	; Return to bank0
	MOVLW	B'10000000'	; RB7 = 1
	MOVWF	PORTB
	RETURN

TMR0_SETUP	
	BANKSEL	OPTION_REG	; Bank1
	MOVLW	B'11100000'	; External clock, rising edge
	MOVWF	OPTION_REG	; Prescaler 1:2
	RETURN

INTERRUPT_SETUP
	BCF	INTCON,T0IF	; Clear overflow flag 
	BSF	INTCON,T0IE	; TMR0 enable
	BCF	INTCON,GIE	; Interrupt disable
	BANKSEL	INTCON	; Bank0
	RETURN
	
	END
