;***********************************************************
; Filename :	lab1001.asm
; Descripton :	Demonstrate Timer1 operation in timer mode 
; MCU :		PIC16F877A
; Copyright : 	2002-2003 Innovative Experiment
;************************************************************
	LIST	P=16F877,W=-302
	INCLUDE	P16F877.INC
	__CONFIG	0x3D32	; HS 20MHz.,WDT-OFF

SECOND	EQU	0x70
MINUTE	EQU	0x71
COUNT	EQU	0x72
BLINK_DP	EQU	0x73

#DEFINE	SW_SET	PORTC,1	; Active low
#DEFINE	SW_START	PORTC,0
#DEFINE	ALARM_ON	BSF   PORTC,2
#DEFINE	ALARM_OFF	BCF   PORTC,2
#DEFINE	SOUND_OFF	BCF   PORTC,3
;***************  MAIN ROUTINE  *****************************
	ORG	0x0000

ORIGIN:	CALL	PORT_SETUP
	CALL	TMR1_INT_SETUP
	CALL	TMR1_SETUP
	GOTO	START	

ISR:	BCF	PIR1,TMR1IF	; Clear overflow flag
	DECFSZ	COUNT,1	; 1 second
	RETFIE
	COMF	BLINK_DP,1
	BTFSS	BLINK_DP,0
	GOTO	CLEAR_DP
SET_DP	BSF	PORTB,7	; Turn-on display
	GOTO	L1
CLEAR_DP	BCF	PORTB,7	; Turn-off display		
L1	DECF	SECOND,1	; 60 seconds
	BZ	LOAD
	MOVLW	.19
	MOVWF	COUNT
	RETFIE
LOAD	DECF	MINUTE,1	; Minutes
	BZ	ALARM
	CALL	SHOWTIME
	MOVWF	PORTB
	MOVLW	.19
	MOVWF	COUNT
	MOVLW	.60
	MOVWF	SECOND
	RETFIE

ALARM	BCF	T1CON,TMR1ON	; Turn-off  TMR1 
	ALARM_ON
	MOVLW	0x3F
	MOVWF	PORTB
SOUND_ON	BSF	PORTC,3	
	CALL	SOUND
	BCF	PORTC,3
	CALL	SOUND
	GOTO	SOUND_ON
	
START:	BTFSC	SW_SET	; Read SW_SET
	GOTO	START
	CALL	SET_TIME
	CALL	DELAY
L2	BTFSC	SW_SET
	GOTO	L3	
	CALL	SET_TIME 
	CALL	DELAY
L3	BTFSC	SW_START	; Read SW_START
	GOTO	L2
OPERATION	BSF	T1CON,TMR1ON	; Start TMR1

	GOTO	$	; Wait for interrupt

;***************  SUBROUTINE  *****************************
PORT_SETUP	CLRF	BLINK_DP
	CLRF	MINUTE	
	BANKSEL	TRISB	; Access port B <bank1>
	CLRF	TRISB	; Port B as output
	MOVLW	B'11110011'	; RC2,RC3 as output
	MOVWF	TRISC	
	BANKSEL	PORTB	; Return to bnak0 	
	CALL	SHOWTIME	; Show '0' on port B
	MOVWF	PORTB
	ALARM_OFF		; Clear alarm
	SOUND_OFF		; Clear sound
	RETURN
	
TMR1_INT_SETUP  NOP
	BANKSEL	PIE1	; Access PIE1 <bank1>	
	BSF	PIE1,TMR1IE	; TMR1 interrupt enable 
	BANKSEL	PIR1	; Access PIR1 <bank0>
	BCF	PIR1,TMR1IF 	; Clear TMR1 overflow flag 
	BSF	INTCON,PEIE	; Peripheral interrupt enable 	BSF	INTCON,GIE	; Global interrupt enable 
	RETURN	

TMR1_SETUP	MOVLW	B'00100000'	; Internal sync clock and prescaler =1:4
	MOVWF	T1CON	; TMR1 off
	CLRF	TMR1H	; Clear for cycle count
	CLRF	TMR1L	;
	MOVLW	.60
	MOVWF	SECOND
	MOVLW	.19
	MOVWF	COUNT
	RETURN

SHOWTIME	MOVFW	MINUTE
	ADDWF	PCL,1
	DT	0x78,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F

SET_TIME	INCF	MINUTE,1
	MOVLW	.10
	SUBWF	MINUTE,0
	BZ	DECREASE
	CALL	SHOWTIME
	MOVWF	PORTB	; Show time
	RETURN
DECREASE	CLRF	MINUTE
	GOTO	SET_TIME

DELAY	CLRF	0x74	; Delay 200 ms
	CLRF	0x75
	MOVLW	.5
	MOVWF	0x76
	DECFSZ	0x74,1
	GOTO	$-1
	DECFSZ	0x75,1
	GOTO	$-3
	DECFSZ	0x76,1
	GOTO	$-5
	RETURN
	
SOUND	CLRF	0x74	; Frequency 650Hz
	MOVLW	.5
	MOVWF	0x75
	DECFSZ	0x74,1
	GOTO	$-1
	DECFSZ	0x75,1
	GOTO	$-3
	RETURN
	
;***************** END OF SUBROUTINES ****************
	END
