;***********************************************************
; Filename :	lab1101.asm
; Descripton :	Demonstrate Timer2 operation in pulse generator 
; MCU :		PIC16F877A
; Copyright : 	2002-2003 Innovative Experiment
;************************************************************
	LIST	P=16F877,W=-302
	__CONFIG	0x3D32

#DEFINE	SWITCH_1	PORTB,0
#DEFINE	SWITCH_2	PORTB,1
#DEFINE	SWITCH_3	PORTB,2

PORTB	EQU	0x06	; Bank0
INTCON	EQU	0x0B
PIR1	EQU	0x0C
TMR2	EQU	0x11
T2CON	EQU	0x12
PR2_TEMP	EQU	0x70	; RAM

TRISB	EQU	0x86	; Bank1
PIE1	EQU	0x8C
PR2	EQU	0x92



GIE	EQU	7
PEIE	EQU	6
TMR2ON	EQU	2
TMR2IF	EQU	1
TMR2IE	EQU	1

	ORG	0x0000
	CALL	PORTB_SETUP
	CALL	INTERRUPT_SETUP
	CALL	TMR2_SETUP1
	GOTO	MAIN

ISR:	COMF	PORTB,1	; Toggle bit
	BCF	PIR1,TMR2IF	; Clear overflow flag
	RETFIE
;***************  SUBROUTINES  ***************
PORTB_SETUP	NOP
	BANKSEL	TRISB	; Bank1
	BCF	TRISB,3	; RB3 = output
	BANKSEL	PORTB	; Bank0
	RETURN

INTERRUPT_SETUP	NOP
	BANKSEL	PIE1	; Bank1
	BSF	PIE1,TMR2IE	; TMR2 interrupt enable
	BANKSEL	INTCON	; Bank0
	BSF	INTCON,PEIE	; Peripheral interrupt enable
	BSF	INTCON,GIE	; Global interrupt enable
	RETURN

TMR2_SETUP1	MOVLW	.124	; Frequency = 1000Hz
	MOVWF	PR2_TEMP
TMR2_SETUP2	MOVLW	B'00100001'	; Stop TMR2,prescaler 1:4,postscaler 1:5
	MOVWF	T2CON	
	BCF	PIR1,TMR2IF	; Clear overflow flag
	CLRF	TMR2	; Clear TMR2 register
	BANKSEL	PR2	; Bank1
	MOVFW	PR2_TEMP	; New frequency
	MOVWF	PR2
	BANKSEL	T2CON	; Bank0
	BSF	T2CON,TMR2ON	; Start TMR2
	RETURN
;************************************************************
MAIN:	BTFSS	SWITCH_1	; Read switch
	GOTO	F1000
	BTFSS	SWITCH_2
	GOTO	F1250
	BTFSS	SWITCH_3
	GOTO	F5000
	GOTO	MAIN

F1000	MOVLW	.124	; Frequency = 1000Hz
	GOTO	CHANGE

F1250	MOVLW	.99	; Frequency = 1250Hz 
	GOTO	CHANGE
	
F5000	MOVLW	.24	; Frequency = 5000 Hz.
CHANGE	MOVWF	PR2_TEMP	
	CALL	TMR2_SETUP2
	GOTO	MAIN

	END
