;***********************************************************
; Filename :	lab1801.asm
; Descripton :	Using ADC and PWM module in light dimmer experiment
; MCU :		PIC16F877(A)
; Copyright : 	2002-2003 Innovative Experiment
;************************************************************
	LIST	P=16F877,W=-302

PORTA	EQU	0x05	; Bank0
PORTC	EQU	0x07
TMR2	EQU	0x11
T2CON	EQU	0x12
CCPR1L	EQU	0x15
CCP1CON	EQU	0x17
ADRESH	EQU	0x1E
ADCON0	EQU	0x1F

TRISA	EQU	0x85	; Bank1
TRISC	EQU	0x87
PR2	EQU	0x92
ADCON1	EQU	0x9F

ADON	EQU	0	; 
GO_DONE	EQU	2
TMR2ON	EQU	2

ACQ_VAR	EQU	0x70	; GP RAM

	__CONFIG	0x3D32	; 20 MHz HS MODE, WDT OFF
	ORG	0x0000

;*************  INITIAL PORT  *****************************  
	BANKSEL	TRISA	; Select TRISA <Bank1>
	BSF	TRISA,0	; Ensure RA0 = analog input
	BCF	TRISC,2	; RC2(CCP1)  = digital output
;***********************  PWM MODULE   ******************************

SET_ADCON1	MOVLW	B'00001110'    ; Left justified and 1 analog channel (RA0)
	MOVWF	ADCON1		
	MOVLW	.249	; Fpwm = 1250 Hz.      
	MOVWF	PR2

	BANKSEL	CCP1CON	; Select CCP1CON <Bank0>
	MOVLW	B'00001111'	; CCP1X:CCP1Y = 00, PWM mode
	MOVWF	CCP1CON	
	CLRF	CCPR1L	; DC = 0 % <start PWM>

	CLRF	TMR2	; Clear  Timer2
	MOVLW	B'00000011'	; Prescaler = 1:16  
	MOVWF	T2CON	; Timer2 off
	BSF	T2CON,TMR2ON   ; Start Timer2

;***********************   ADC MODULE   *****************************

	MOVLW	B'10000000'    ; ADC Clock = Fosc/32 @20 MHz.,  
	MOVWF	ADCON0	; Channel0 (RA0)
NEXT	BSF	ADCON0,ADON	; Turn on ADC module
	CALL	ACQ_TIME 	; Delay for acquisition time
	BSF	ADCON0,GO_DONE ; Start conversion 
	BTFSC	ADCON0,GO_DONE ; Wait for comnversion complete    
	GOTO	$-1
	BCF	ADCON0,ADON	; Disable ADC module
	
	MOVFW	ADRESH	; Load upper 8-bit into CCPR1L
	MOVWF	CCPR1L	; Neglect CCP1X:CCP1Y <00>
		
	GOTO	NEXT  	; ** Next conversion **      

ACQ_TIME	MOVLW	.33	; Delay ~20 microsecond 
	MOVWF	ACQ_VAR
	DECFSZ	ACQ_VAR,1
	GOTO	$-1
	RETURN	

	END
