;***********************************************************
; Filename :	lab1901.asm
; Descripton :	Si0mple data logger 
; MCU :		PIC16F877(A)
; Copyright : 	2002-2003 Innovative Experiment
;************************************************************
	LIST	P=16F877,W=-302
	__CONFIG	0x3D32	; HS-20 MHZ, WDT OFF

PCL	EQU	0x02	; Bank 0
INTCON	EQU	0x0B	
PIR1	EQU	0x0C	
RCSTA	EQU	0x18
TXREG	EQU	0x19
RCREG	EQU	0x1A
ADRESH	EQU	0x1E
ADCON0	EQU	0x1F	

TRISC	EQU	0x87	; Bank1
PIE1	EQU	0x8C
TXSTA	EQU	0x98
SPBRG	EQU	0x99
ADRESL	EQU	0x9E
ADCON1	EQU	0x9F

DELAY_VAR	EQU	0x70	; Common RAM
TEMP0	EQU	0x71
TEMP1	EQU	0x72
COUNT1	EQU	0x73
COUNT2	EQU	0x74
COUNT3	EQU	0x75

ADON	EQU	0	; Bit
BRGH	EQU	2	
GO_DONE	EQU	2
CHS0	EQU	3
CHS1	EQU	4
TXIF	EQU	4
CREN	EQU	4
RCIE	EQU	5
RCIF	EQU	5
TXEN	EQU	5
PEIE	EQU	6
SPEN	EQU	7
GIE	EQU	7
ADFM	EQU	7

LF	EQU	0x0A 	; Line feed 
FF	EQU	0x0C	; Form feed
CR	EQU	0x0D 	; Carriage return

;************************************************************************************
	ORG	0x0000
	GOTO	START	; Reset vector

	ORG	0x0004	; Interrupt vector
	GOTO	INTERRUPT

;***********************  SET UP ADC AND USART MODULE  ******************************
START:	NOP
	BANKSEL	TRISC	; Select TRISC <bank1>
	BSF	TRISC,6	; RB2 should be set as input (TX pin)	

SETUP_ADCON1	MOVLW	B'00000000' 	; Left justified, 8 channels
	MOVWF	ADCON1	; but use 4 channels only

SETUP_USART	BSF	PIE1,RCIE	; Receive interrupt enable
	BSF	INTCON,PEIE 	; Peripheral interrupt enable				
	MOVLW	.64
	MOVWF	SPBRG	; Baudrate = 19200 bit per second
	BSF 	TXSTA,BRGH	; Use high speed baudrate
	BSF	TXSTA,TXEN	; Transmit enable

	BANKSEL	INTCON	; Select INTCON <bank0>
	BSF	INTCON,GIE	; Interrupt enable
	BSF	RCSTA,CREN	; Receive  enable
	BSF	RCSTA,SPEN  	; Serial port enable 
	CALL	SHOW_MESSAGE
	MOVLW	0x30	; Default CH0 <RA0> 
	MOVWF	TEMP0
	MOVWF	TEMP1	
;*************************************************************************************		
ADC	CALL	SETUP_ADCON0
	BSF	ADCON0,ADON	; Turn on ADC module
	CALL	ACQ_TIME 	; Delay for acquisition time
	BSF	ADCON0,GO_DONE  ; Start conversion 
WAIT	BTFSC	ADCON0,GO_DONE  ; Wait for complete    
	GOTO	WAIT
	BCF	ADCON0,ADON	; Turn off ADC module

	MOVLW	"C"
	CALL	TX
	MOVLW	"H"
	CALL	TX
	MOVFW	TEMP0
	CALL	TX
	MOVLW	"="
	CALL	TX

	MOVLW	.8
	MOVWF	COUNT3	; Rotate 8 times
ROTATE1	RLF	ADRESH,1	; Show result high
	BC	CARRY_SET
CARRY_CLEAR	MOVLW	"0"
	CALL	TX
	GOTO	LOOP2	
CARRY_SET	MOVLW	"1"
	CALL	TX
LOOP2	DECFSZ	COUNT3,1
	GOTO	ROTATE1
	BANKSEL	ADRESL	; Select ADRESL <bank1>
	MOVFW	ADRESL	; Show result low
	MOVWF	COUNT3
	BANKSEL	ADCON0	; Return to bank0

	RLF	COUNT3,1
	BC	ROTATE2
	MOVLW	"0"
	CALL	TX
	GOTO	LOOP3
ROTATE2	MOVLW	"1"
	CALL	TX
LOOP3	RLF	COUNT3,1
	BC	ROTATE3
	MOVLW	"0"
	CALL	TX
	GOTO	LOOP4
ROTATE3	MOVLW	"1"
	CALL	TX
LOOP4	MOVLW	CR
	CALL	TX
	
	GOTO	ADC
		
;******************************  SUBROUTINE  ********************************
SHOW_MESSAGE	MOVLW	.90	; Number of characters 
	MOVWF	COUNT1
	CLRF	COUNT2
LOOP1	MOVFW	COUNT2
	CALL	TABLE
	CALL	TX
	INCF	COUNT2,1
	DECFSZ	COUNT1,1
	GOTO	LOOP1
	RETURN	
TABLE	ADDWF	PCL,1
	DT	FF,"Full duplex communication",CR,LF
	DT	"ADC MONITOR",LF,CR
	DT	"Press 0,1,2,3 for CH0,CH1,CH2,CH3 respectively",LF,LF,CR

TX	BTFSS	PIR1,TXIF	; Skip if TXREG empty
	GOTO	$-1	; Not empty then wait
	MOVWF	TXREG	; Send data to RS-232	
	RETURN

ACQ_TIME	MOVLW	.33	; Delay 20 microsecond
	MOVWF	DELAY_VAR
	DECFSZ	DELAY_VAR,1
	GOTO	$-1
	GOTO	$+1
	RETURN

INTERRUPT	NOP
	BANKSEL	RCREG	; Select RCREG <bank0>
	MOVFW	RCREG	; Read buffer1
	MOVFW	RCREG	; Read buffer2
	MOVWF	TEMP0

FIND_CHANNEL	MOVLW	0x30
	SUBWF	TEMP0,0
	BZ	CHANNEL_OK	; CH0 request <RA0> 

	MOVLW	0x31
	SUBWF	TEMP0,0
	BZ	CHANNEL_OK	; CH1 request <RA1>

	MOVLW	0x32
	SUBWF	TEMP0,0
	BZ	CHANNEL_OK	; CH2 request <RA2>

	MOVLW	0x33
	SUBWF	TEMP0,0
	BZ    	CHANNEL_OK	; CH4 request <RA3>
	MOVFW	TEMP1
	MOVWF	TEMP0
	RETFIE		
CHANNEL_OK	MOVFW	TEMP0
	MOVWF	TEMP1
	CALL	SHOW_MESSAGE
SETUP_ADCON0    MOVLW	B'10000000' 	; Tad = 1.6uS @20 MHz.
	MOVWF	ADCON0
	BTFSC	TEMP0,0	; Select ADC channel
	BSF	ADCON0,CHS0
	BTFSC	TEMP0,1
	BSF	ADCON0,CHS1
	RETFIE

	END
