;********************************************************************
; Filename :	lab2301.asm
; Description :	Digital thermostat with DS1620 and MC14489 
;			controlled by PIC16F877
; MCU :		PIC16F877(A)
; Copyright 2002-2003 Innovative Experiment Co.,Ltd. Thailand
;********************************************************************
	LIST	P=16F877,W=-302
	INCLUDE	P16F877.INC
	__CONFIG	0x3D32	; 20 MHz.,HS MODE
       CBLOCK	0x70	; COMMON RAM
          COUNT1,COUNT2,H_BYTE,L_BYTE,COUNT,TEMP,HALF,R0,R1,R2,R3,R4
           ENDC

#DEFINE	ENABLE_DS1620	BSF  PORTC,1	; Active high
#DEFINE	DISABLE_DS1620	BCF  PORTC,1	; Active low
#DEFINE	ENABLE_MC14489  BCF  PORTC,0	; Active low
#DEFINE	DISABLE_MC14489	BSF  PORTC,0	; Active high 
              
            	ORG	0x0000	
            	
PORT_SETUP	DISABLE_DS1620		; Disable DS1620
	DISABLE_MC14489		; Disable MC14489
	BANKSEL	TRISC	; Bank1
	MOVLW	B'11010100'	; RC5,RC3,RC1,RC0 = output
	MOVWF	TRISC

	CALL	DS1620_INIT	
DS1620_SETUP	MOVLW	0x0C	; Config command 
	CALL	TX1620
CONFIG_DATA	MOVLW	0x0A	; Continous conversion
	CALL	TX1620	
	CALL	DELAY	; Delay 10 ms
	DISABLE_DS1620
WRITE_TH	MOVLW	0x01	
	CALL	TX1620
	MOVLW	.66	; Set LSB data of TH : 33oC   
	CALL	TX1620
	MOVLW	.00	; Set MSB data of TH : 33oC
	CALL	TX1620
	CALL	DELAY	; Delay 10 ms
	DISABLE_DS1620
WRITE_TL	MOVLW	0x02	
	CALL	TX1620
	MOVLW	.56	; Set LSB data of TL : 28oC
	CALL	TX1620
	MOVLW	.00	; Set MSB data of TL : 28oC
	CALL	TX1620
	CALL	DELAY	; Delay 10 ms
	DISABLE_DS1620
	GOTO	START	

READ_TEMP	CALL	DS1620_INIT
START	MOVLW	0xEE
	CALL	TX1620	; Start conversion
	CALL	DELAY	; Delay 10 ms
	DISABLE_DS1620	
	MOVLW	0xAA	; Read command
	CALL	TX1620
	CALL	TX1620	; Read LSB
	CALL	REVERSE
	CLRF	H_BYTE
	MOVWF	L_BYTE
	CLRF	HALF
	CLRC
	RRF	L_BYTE,1	; F/2
	BNC	LOOP2
	MOVLW	.5
	MOVWF	HALF
LOOP2	CALL 	BCD
	CALL	TX1620	; Read MSB
	DISABLE_DS1620		; Disable DS1620

MC14489_SETUP	;
	BANKSEL	SSPSTAT	; Bank1
	BSF	SSPSTAT,CKE	; CKE=1 (first edge/rising edge)
BANKSEL	SSPCON	; Bank0
	MOVLW	B'00000001'	; CLK=1.25 MHz.,CKP=0 (idle clock low)
	MOVWF	SSPCON		
	MOVLW	B'11000001'	; Setup configuration
	CALL 	TX14489
	DISABLE_MC14489	
	MOVLW	B'10100000'	; DP:Digit5
	CALL	TX14489		
	SWAPF	R2,0	; Digit4:Digit3
	CALL	TX14489
	SWAPF	R2,0	; Digit2:Digit1
	ANDLW	B'11110000'	; Mask bit
	IORWF	HALF,0	
	CALL	TX14489
	DISABLE_MC14489
	GOTO	READ_TEMP
	
;*******************************   SUBROUTINES   *****************
DS1620_INIT	; Initial DS1620
	BANKSEL	SSPSTAT	; Bank1
	BCF	SSPSTAT,CKE	; CKE=0 (Second edge/falling edge)
	BANKSEL	SSPCON	; Bank0
	MOVLW	B'00010001'	; CLK=1.25 MHz.,CKP=1 (idle clock high)
	MOVWF	SSPCON
	RETURN

TX1620	CALL	REVERSE
	ENABLE_DS1620		; Enable DS1620

	GOTO	TX

TX14489	ENABLE_MC14489		; Enable MC14489

TX	BSF	SSPCON,SSPEN	; Enable master 
	MOVWF	SSPBUF
	BANKSEL	SSPSTAT	; Bank1
	BTFSS	SSPSTAT,BF
	GOTO	$-1	
	BANKSEL	SSPBUF	; Bank0
	MOVFW	SSPBUF	; Read SSPBUF for clearing buffer
	RETURN

DELAY	CLRF	0x20	; Delay 10 ms
	MOVLW	.65
	MOVWF	0x21
	DECFSZ	0x20,F
	GOTO	$-1
	DECFSZ	0x21,F
	GOTO	$-3
	RETURN

REVERSE	MOVWF	R3	
	MOVLW	.8	
	MOVWF	COUNT2
LOOP1	RRF	R3,F
	RLF	R4,F
	DECFSZ	COUNT2,F
	GOTO	LOOP1
	MOVFW	R4	 
	RETURN	

BCD  	CLRC	
	MOVLW   .16
	MOVWF   COUNT
	CLRF    R0
	CLRF    R1
	CLRF    R2
LOOP16  	RLF     L_BYTE,1
	RLF     H_BYTE,1
	RLF     R2,1
	RLF     R1,1
	RLF     R0,1
	DECFSZ  COUNT,1
	GOTO    ADJDEC
	RETLW   0x00
ADJDEC  	MOVLW   R2	; Adjust exact data to decimal value routine
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R1
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R0
	MOVWF   FSR
	CALL    ADJBCD
	GOTO    LOOP16
ADJBCD  	MOVLW   0x03	; Adjust data to BCD routine
	ADDWF   INDF,0
	MOVWF   TEMP
	BTFSC   TEMP,3          
	MOVWF   INDF
	MOVLW   0x30
	ADDWF   INDF,0
	MOVWF   TEMP
	BTFSC   TEMP,7          
	MOVWF   INDF            
	RETLW   0x00

	END
;********************************************************************
