;***********************************************************
; Filename :	lab2501.asm
; Descripton :	Demonstrate using SSP module in I2C master mode for interface DS1307 real-time clock 
; MCU :		PIC16F877 (A)
; Copyright : 	2002-2003 Innovative Experiment
;************************************************************
	LIST	P=16F877,W=-302
	INCLUDE	P16F877.INC
	__CONFIG	0x3D7A

         CBLOCK		0x50	
            AA,BB,CC,NUM_1,NUM_2,COUNT,XX,YY,TEMP1,SWTICH,CURSOR
             ENDC

         CBLOCK	 	0x70
           SECONDS,MINUTES,HOURS,DAY,DATE,MONTH,YEAR
             ENDC


#DEFINE	DATA_BUS	PORTB	
#DEFINE	LOW_E	BCF PORTA,0
#DEFINE	HIGH_E	BSF PORTA,0
#DEFINE	LOW_RS      	BCF PORTA,1
#DEFINE	HIGH_RS	BSF PORTA,1
#DEFINE	SW_SET	PORTD,2
#DEFINE	SW_SHIFT	PORTD,1
#DEFINE	SW_UP	PORTD,0
		
      CONSTANT	FUNCTION_VALUE  = B'00111000'
      CONSTANT	ENTRY_VALUE     = B'00000110'
      CONSTANT	DISPLAY_VALUE   = B'00001100'

;************************************************************************************
	ORG	0x0000		

	GOTO	MAIN_PROG

INITIAL:	MOVLW	.7	; Initial port and register
	BANKSEL	ADCON1	; Bank1
	MOVWF	ADCON1	; RA3-RA0 = digital port
	CALL	DISABLE	; Disable after POR	
	MOVLW	B'11111100'
	MOVWF	TRISA	; RA1-RA0 = output
	CLRF	TRISB	; PORTB   = output
	BSF	TRISC,3	; RC3 = input only
	BSF	TRISC,4	; RC4 = inpuit onlt
	MOVLW	.49	; SCL clock = 100 kHz
	MOVWF	SSPADD	
	BSF	SSPSTAT,SMP	; Standard mode	
	BANKSEL	SSPCON	; Bank0
	MOVLW	B'00001000'	; I2C master mode
	MOVWF	SSPCON		
	BSF	SSPCON,SSPEN	; SSP enable
INIT_LCD:	CALL	DELAY15M	; Wait for internal reset
	CALL	INITIAL_LCD 	; Set-up LCD
	CALL	CLEAR_SCREEN	; Clear display
ASSIGN_CHAR:	MOVLW	.21	; Fill character
	MOVWF	COUNT
	MOVLW	0x20	; address 0x20
	MOVWF	FSR	; POINTER = D'0'
	CLRF	TEMP1
LOOP1	MOVFW	TEMP1	
	CALL	LOOP2
	MOVWF	INDF	; Write to table
	INCF	TEMP1,1
	INCF	FSR,1
	DECFSZ	COUNT,1
	GOTO	LOOP1
	RETURN
LOOP2	ADDWF	PCL,1	
	DT	"MONTUEWEDTHUFRISATSUN"  ; DAY=>1,2,3,4,5,6,7

DAY_TABLE:	ANDLW	B'00000111'	; Mask bit
	ADDLW	-1	; W = W-1
	ADDWF	PCL,1
	GOTO	MON_TABLE
	GOTO	TUE_TABLE
	GOTO	WED_TABLE
	GOTO	THU_TABLE
	GOTO	FRI_TABLE
	GOTO	SAT_TABLE
	GOTO	SUN_TABLE
MON_TABLE	MOVLW	0x20+.0	
	GOTO	GET_CHAR
TUE_TABLE	MOVLW	0x20+.3	
	GOTO	GET_CHAR
WED_TABLE	MOVLW	0x20+.6	
	GOTO	GET_CHAR
THU_TABLE	MOVLW	0x20+.9	
	GOTO	GET_CHAR
FRI_TABLE	MOVLW	0x20+.12	
	GOTO	GET_CHAR
SAT_TABLE	MOVLW	0x20+.15	
	GOTO	GET_CHAR
SUN_TABLE	MOVLW	0x20+.18	
GET_CHAR	MOVWF	FSR	; FSR = pointer 
	MOVLW	.3
	MOVWF	COUNT
LOOP3 	MOVFW	INDF
	CALL	WRITE_DATA
	INCF	FSR,1
	DECFSZ	COUNT,1	
	GOTO	LOOP3	
	RETURN	

;*****************************************************		

MAIN_PROG	CALL	INITIAL

MAIN_LOOP	CALL	SHOW
	CALL	READ_SWITCH
	GOTO	MAIN_LOOP

READ_SWITCH	BTFSC	SW_SET	; Skip if pressed
	RETURN
	MOVLW	.25
	CALL	DELAY_?
	CALL	CURSOR_ON
SW_LOOP	BTFSC	SW_SHIFT
	GOTO	SW_UP_LOOP
	MOVLW	.12
	CALL	DELAY_?
	CALL	SHIFT_CURSOR		
SW_UP_LOOP	BTFSC	SW_UP
	GOTO	ENTER
	MOVLW	.7
	CALL	DELAY_?
	CALL	INCREASE
ENTER	BTFSC	SW_SET
	GOTO	SW_LOOP
	CALL	CURSOR_OFF
	CALL	WRITE_DS1307
	MOVLW	.25
	CALL	DELAY_?
	RETURN

INCREASE	MOVLW	0x82
	SUBWF	CURSOR,0
	SKPZ		; Skip if "DAY" position
	GOTO	NEXT1
	MOVLW	B'00000111'
	ANDWF	DAY,1
	INCF	DAY,1
	BTFSC	DAY,3	; Skip if DAY < 8
	CALL	DAY_ADJ	; Set to 1 <Monday>
	MOVLW	0x80	; Set cursor position 
	CALL	SET_POSITION
	MOVFW	DAY
	CALL	DAY_TABLE
	MOVLW	0x82
	MOVWF	CURSOR
	CALL	SET_POSITION
	RETURN
DAY_ADJ	MOVLW	0x01
	MOVWF	DAY
	RETURN

NEXT1	MOVLW	0x85
	SUBWF	CURSOR,0
	SKPZ		; Skip if "DATE" position
	GOTO	NEXT2
	CALL	DATE_ADD
	MOVLW	0x84	; Set position
	CALL	SET_POSITION	
	MOVFW	DATE
	CALL	ADJUST
	MOVLW	0x85
	MOVWF	CURSOR
	CALL	SET_POSITION
	RETURN

DATE_ADD	MOVLW	0x31
	SUBWF	DATE,0
	SKPNZ
	CLRF	DATE	
	MOVFW	DATE
	MOVWF	NUM_2
	MOVLW	0x01
	MOVWF	NUM_1
	CALL	BCDADD
	MOVFW	NUM_2	; W = result
	MOVWF	DATE
	RETURN

NEXT2	MOVLW	0x88
	SUBWF	CURSOR,0
	SKPZ		; Skip if "MONTH" position
	GOTO	NEXT3
	CALL	MONTH_ADD
	MOVLW	0x87	; Set position
	CALL	SET_POSITION	
	MOVFW	MONTH
	CALL	ADJUST
	MOVLW	0x88
	MOVWF	CURSOR
	CALL	SET_POSITION
	RETURN

MONTH_ADD	MOVLW	0x12
	SUBWF	MONTH,0
	SKPNZ
	CLRF	MONTH
	MOVFW	MONTH
	MOVWF	NUM_2
	MOVLW	0x01
	MOVWF	NUM_1
	CALL	BCDADD
	MOVFW	NUM_2	; W = result
	MOVWF	MONTH
	RETURN

NEXT3	MOVLW	0x8B
	SUBWF	CURSOR,0
	SKPZ		; Skip if "YEAR" position
	GOTO	NEXT4
	CALL	YEAR_ADD
	MOVLW	0x8A	; Set position
	CALL	SET_POSITION	
	MOVFW	YEAR
	CALL	ADJUST
	MOVLW	0x8B
	MOVWF	CURSOR
	CALL	SET_POSITION
	RETURN

YEAR_ADD	MOVLW	0x99
	SUBWF	YEAR,0
	SKPNZ
	GOTO	YEAR_ADJ
	MOVFW	YEAR
	MOVWF	NUM_2
YEAR_LOOP	MOVLW	0x01
	MOVWF	NUM_1
	CALL	BCDADD
	MOVFW	NUM_2	; W = result
	MOVWF	YEAR
	RETURN
YEAR_ADJ	MOVLW	0x99
	MOVWF	NUM_2
	GOTO	YEAR_LOOP

NEXT4	MOVLW	0xC5
	SUBWF	CURSOR,0
	SKPZ		; Skip if "HOUR" position
	GOTO	NEXT5
	CALL	HOURS_ADD
	MOVLW	0xC4	; Set position
	CALL	SET_POSITION	
	MOVFW	HOURS
	CALL	ADJUST
	MOVLW	0xC5
	MOVWF	CURSOR
	CALL	SET_POSITION
	RETURN

HOURS_ADD	MOVLW	0x23
	SUBWF	HOURS,0
	SKPNZ
	GOTO	HOURS_ADJ
	MOVFW	HOURS
	MOVWF	NUM_2
HOURS_LOOP	MOVLW	0x01
	MOVWF	NUM_1
	CALL	BCDADD
	MOVFW	NUM_2	; W = result
	MOVWF	HOURS
	RETURN
HOURS_ADJ	MOVLW	0x99
	MOVWF	NUM_2
	GOTO	HOURS_LOOP

NEXT5	MOVLW	0xC8
	SUBWF	CURSOR,0
	SKPZ		; Skip if "MINUTE" position
	GOTO	NEXT6
	CALL	MINUTES_ADD
	MOVLW	0xC7	; Set position
	CALL	SET_POSITION	
	MOVFW	MINUTES
	CALL	ADJUST
	MOVLW	0xC8
	MOVWF	CURSOR
	CALL	SET_POSITION
	RETURN

MINUTES_ADD	MOVLW	0x59
	SUBWF	MINUTES,0
	SKPNZ
	GOTO	MINUTES_ADJ
	MOVFW	MINUTES
	MOVWF	NUM_2
MINUTES_LOOP	MOVLW	0x01
	MOVWF	NUM_1
	CALL	BCDADD
	MOVFW	NUM_2	; W = result
	MOVWF	MINUTES
	RETURN
MINUTES_ADJ	MOVLW	0x99
	MOVWF	NUM_2
	GOTO	MINUTES_LOOP

NEXT6	MOVLW	0xCB
	SUBWF	CURSOR,0
	SKPZ		; Skip if "SECOND" position
	GOTO	INCREASE
	CALL	SECONDS_ADD
	MOVLW	0xCA	; Set position
	CALL	SET_POSITION	
	MOVFW	SECONDS
	CALL	ADJUST
	MOVLW	0xCB
	MOVWF	CURSOR
	CALL	SET_POSITION
	RETURN

SECONDS_ADD	MOVLW	0x59
	SUBWF	SECONDS,0
	SKPNZ
	GOTO	SECONDS_ADJ
	MOVFW	SECONDS
	MOVWF	NUM_2
SECONDS_LOOP	MOVLW	0x01
	MOVWF	NUM_1
	CALL	BCDADD
	MOVFW	NUM_2	; W = result
	MOVWF	SECONDS
	RETURN
SECONDS_ADJ	MOVLW	0x99
	MOVWF	NUM_2
	GOTO	SECONDS_LOOP

CURSOR_ON	MOVLW	B'00001101'	; Cursor enable
	CALL	WRITE_COMMAND
	MOVLW	0x82
	MOVWF	CURSOR	; Cursor position 
	CALL	SET_POSITION
	RETURN

CURSOR_OFF	MOVLW	B'00001100'	; Cursor disable
	CALL	DISPLAY_CONTROL
	RETURN

SHIFT_CURSOR	BTFSC	CURSOR,6	; Skip if line 1
	GOTO	SHIFT2	; Line 2
SHIFT1	MOVLW	0x03
	ADDWF	CURSOR,1
	MOVLW	0x8E
	SUBWF	CURSOR,0
	BZ	CURSOR_ADJ1	; Branch if cursor = 0x8E
	MOVFW	CURSOR
	CALL	SET_POSITION
	RETURN
CURSOR_ADJ1	MOVLW	0xC5	
	MOVWF	CURSOR	
	CALL	SET_POSITION
	RETURN	
SHIFT2	MOVLW	0x03
	ADDWF	CURSOR,1
	MOVLW	0xCE
	SUBWF	CURSOR,0
	BZ	CURSOR_ADJ2	; Branch if cursor = 0xCE
	MOVFW	CURSOR
	CALL	SET_POSITION
	RETURN
CURSOR_ADJ2	MOVLW	0x82
	MOVWF	CURSOR
	CALL	SET_POSITION
	RETURN

;******************** SUBROUTINES  ********************
SHOW:	
READ_DS1307:	CALL	START_I2C	; Start slave
	MOVLW	0xD0	; DS1307 write address
	CALL	WRITE_I2C
	MOVLW	0x00	; Point to address 0x00
	CALL	WRITE_I2C
	CALL	STOP_I2C
	CALL	START_I2C
	MOVLW	0xD1	; DS1307 read address
	CALL	WRITE_I2C	
	CALL	RD_SEND_ACK	; Read SECOND
	MOVWF	SECONDS
	CALL	RD_SEND_ACK	; Read MINUTE
	MOVWF	MINUTES
 	CALL	RD_SEND_ACK	; Read HOUR
	MOVWF	HOURS
	CALL	RD_SEND_ACK	; Read DAY
	MOVWF	DAY
	CALL	RD_SEND_ACK	; Read DATE
	MOVWF	DATE
 	CALL	RD_SEND_ACK	; Read MONTH
	MOVWF	MONTH
 	CALL	RD_SEND_NACK	; Read YEAR
	MOVWF	YEAR
	CALL	STOP_I2C	; Stop slave
OSD:	MOVLW	0x80
	CALL	SET_POSITION	; LCD position
	MOVFW	DAY
	CALL	DAY_TABLE
 	MOVLW	' '
 	CALL	WRITE_DATA
 	MOVFW	DATE
 	CALL	ADJUST
 	MOVLW	'-'
 	CALL	WRITE_DATA
 	MOVFW	MONTH
 	CALL	ADJUST
 	MOVLW	'-'
 	CALL	WRITE_DATA
	MOVFW	YEAR
 	CALL	ADJUST

	MOVLW	0xC4
	CALL	SET_POSITION		
	MOVFW	HOURS
 	CALL	ADJUST
 	MOVLW	':'
 	CALL	WRITE_DATA
	MOVFW	MINUTES
 	CALL	ADJUST
 	MOVLW	':'
 	CALL	WRITE_DATA
	MOVFW	SECONDS
 	CALL	ADJUST
	RETURN
START_I2C:	
	BANKSEL	SSPCON2	; Bank1
	BSF	SSPCON2,SEN	; I2C start
	CALL	CHECK_SSPIF	; Wait for start completed
	RETURN

WRITE_I2C:	
	BANKSEL	SSPBUF	; Bank0
	MOVWF	SSPBUF	; Write address or data
	CALL	CHECK_SSPIF	; Wait for write completed	
	BANKSEL	SSPCON2	; Bank1
	BTFSS	SSPCON2,ACKSTAT	; Skip if NO ACK
	RETURN	
	CALL	STOP_I2C
	GOTO	$	; Endless loop if NO ACK

RD_SEND_ACK:	
	BANKSEL	SSPCON2	; Bank1
	BSF	SSPCON2,RCEN	; Master receive enable
	CALL	CHECK_SSPIF	; Wait for reading completed
	MOVFW	SSPBUF	; Read data into W register
	CALL	SEND_ACK
	RETURN

RD_SEND_NACK:	
	BANKSEL	SSPCON2	; Bank1
	BSF	SSPCON2,RCEN	; Master receive enable 
	CALL	CHECK_SSPIF	; Wait for reading completed 
	MOVFW	SSPBUF	; Read data into W register 
	CALL	SEND_NACK
	RETURN

CHECK_SSPIF:	
	BANKSEL	PIR1	; Bank0
	BTFSS	PIR1,SSPIF	; Skip if start completed
	GOTO	$-1
	BCF	PIR1,SSPIF	; Clear SSPIF flag
	RETURN
SEND_ACK:	
	BANKSEL	SSPCON2	; Bank1
	BCF	SSPCON2,ACKDT	; Send ACK (send "0")
	BSF	SSPCON2,ACKEN	; ACK. enable
	CALL	CHECK_SSPIF	; Wait for sending completed
	RETURN	
SEND_NACK:	
	BANKSEL	SSPCON2	; Bank1
	BSF	SSPCON2,ACKDT	; Send NACK (send "1")
	BSF	SSPCON2,ACKEN	; ACK. enable
	CALL	CHECK_SSPIF	Wait for sending completed 
	RETURN	

STOP_I2C:	
	BANKSEL	SSPCON2	; Bank1
	BSF	SSPCON2,PEN	; Stop I2C,release SCL & SDA line
	CALL	CHECK_SSPIF	; Wait for stop completed
	RETURN	
WRITE_DS1307:	CALL	START_I2C	; Start slave
	MOVLW	0xD0	; DS1307 write address
	CALL	WRITE_I2C	

	MOVLW	0x00	; Point to address 0x00 <SECONDS register>
	CALL	WRITE_I2C

_TIME	MOVFW	SECONDS	; Set SECONDS 
	CALL	WRITE_I2C
	MOVFW	MINUTES	; Set MINUTES 
	CALL	WRITE_I2C  
	MOVFW	HOURS	; Set HOURS <24 HR>
	CALL	WRITE_I2C

_DATE	MOVFW	DAY	; Set DAY 
	CALL	WRITE_I2C
	MOVFW	DATE	; Set DATE
	CALL	WRITE_I2C	
	MOVFW	MONTH	; Set MONTH
	CALL	WRITE_I2C	 
	MOVFW	YEAR	; Set YEAR
	CALL	WRITE_I2C	
	CALL	STOP_I2C
	RETURN
ADJUST	MOVWF	TEMP1	; TEMP1 = XY
	SWAPF	TEMP1,0	; W     = YX
	CALL	ADJUST2	
	MOVFW	TEMP1
ADJUST2	ANDLW	B'00001111'	; Mask bit
	ADDLW	0x30
	CALL	WRITE_DATA
	RETURN

INITIAL_LCD	CALL	FUNCTION_SET	
	CALL	FUNCTION_SET	
	CALL	FUNCTION_SET	
	CALL	FUNCTION_SET	
	CALL	ENTRY_MODE_SET
	CALL	DISPLAY_CONTROL
	RETURN
FUNCTION_SET	MOVLW	FUNCTION_VALUE
	GOTO	WRITE_COMMAND2
ENTRY_MODE_SET	MOVLW	ENTRY_VALUE
	GOTO	WRITE_COMMAND
DISPLAY_CONTROL	MOVLW	DISPLAY_VALUE
	GOTO	WRITE_COMMAND
WRITE_DATA	HIGH_RS
	GOTO	WRITE
WRITE_COMMAND	LOW_RS
WRITE	MOVWF	DATA_BUS
	NOP
	NOP
	NOP
	NOP
	LOW_E		;Enable pulse
	CALL	DELAY40U
DISABLE	HIGH_E
	NOP
	NOP
	NOP
	NOP
	RETURN
WRITE_COMMAND2	LOW_RS
	MOVWF	DATA_BUS
	NOP
	NOP
	NOP
	NOP
	LOW_E		; Enable pulse
	CALL	DELAY5M
	HIGH_E
	RETURN
CLEAR_SCREEN	MOVLW	B'00000001'	
	GOTO	WRITE_COMMAND2
SET_POSITION	GOTO	WRITE_COMMAND	
DELAY5M	MOVLW	.35	; Delay ~ 5.4 millisecond
	GOTO	DELAY	
DELAY15M	MOVLW	.100	; Delay ~ 15 millisecond
DELAY	MOVWF	XX	; For internal reset
	CLRF	YY
	DECFSZ	YY,1
	GOTO	$-1
	DECFSZ	XX,1
	GOTO	$-3
	RETURN
DELAY40U	MOVLW	.60	; Delay ~ 40 microsecond
MOVWF	XX
	DECFSZ	XX,1
	GOTO	$-1
	RETURN

DELAY_?	CLRF	AA
	CLRF	BB
	MOVWF	CC
	DECFSZ	AA,1
	GOTO	$-1
	DECFSZ	BB,1
	GOTO	$-3
	DECFSZ	CC,1
	GOTO	$-5
	RETURN

BCDADD  	MOVFW    	NUM_1
	ADDWF   	NUM_2,1      
	CLRF    	NUM_1
	RLF     	NUM_1,1
	SKPNDC
	GOTO    	_ADJUST       
	MOVLW   	0x06
	ADDWF   	NUM_2,1      
	SKPNC
	INCF    	NUM_1,1
	SKPDC
	SUBWF   	NUM_2,1 
	GOTO    	OVER1
_ADJUST  	MOVLW   	0x06
	ADDWF   	NUM_2,1 
OVER1   	MOVLW   	0x60    
	ADDWF   	NUM_2,1
	SKPNC	
	GOTO    	OVER3
	BTFSS   	NUM_1,0
	SUBWF   	NUM_2,1
	RETLW   	0x00
OVER3   	MOVLW   	0x01
	MOVWF   	NUM_1
	RETURN	

	END
