;***********************************************************
; Filename :	lab2601.asm
; Descripton :	Demonstrate PIC16F877 interface with DS1820 for temperature measuring 
; MCU :		PIC16F877 (A)
; Copyright : 	2002-2003 Innovative Experiment
;************************************************************
	LIST	P=16F877,W=-302 
	INCLUDE	P16F877.INC
	__CONFIG	0x3D32	; XTAL 20 MHZ. HS GAIN
  CBLOCK	0x20		; RAM in bank0
     TEMP_WR,TEMP_RD,COUNT,COUNT8,DP,SIGN,BUFFER0,BUFFER1
       TEMP,H_BYTE,L_BYTE,R0,R1,R2,XX,YY
         ENDC

	ORG	.0000000

ORIGIN 	CALL	INIT_LCD 	; Setup LCD
	MOVLW	"T"	; Setup message
	CALL	WR_LCD_DATA
	MOVLW	"e"
	CALL	WR_LCD_DATA
	MOVLW	"m"
	CALL	WR_LCD_DATA
	MOVLW	"p"
	CALL	WR_LCD_DATA
	MOVLW	"="
	CALL	WR_LCD_DATA
	MOVLW	" "
	CALL	WR_LCD_DATA

REPEAT	CALL	CONVERT_TEMP
	CALL	READ_TEMP
	CALL	SHOW_TEMP
	GOTO	REPEAT

;*********************************** DS1820 ***************************************
#DEFINE	DQ_PIN	PORTB,0	; RB0 = DQ pin

CONVERT_TEMP	CALL	RESET_PULSE
	CALL	PRESENCE_PULSE
	MOVLW	0xCC	; Skip ROM 
	CALL	OW_WRITE
	MOVLW	0x44	; Convert temperature
	CALL	OW_WRITE
	RETURN
READ_TEMP	CALL	RESET_PULSE
	CALL	PRESENCE_PULSE
	MOVLW	0xCC	; Skip ROM
	CALL	OW_WRITE
	MOVLW	0xBE	; Read temperature <LSB>
	CALL	OW_WRITE
	CALL	OW_READ
	RETURN

SHOW_TEMP	CALL	CONVERT_VALUE
	MOVLW	0x86	; Set DDRAM position
	CALL	SET_POSITION
	MOVFW	SIGN 
	CALL	WR_LCD_DATA
	MOVFW	R0 
	CALL	WR_LCD_DATA
	MOVFW	R1 
	CALL	WR_LCD_DATA
	MOVFW	R2 
	CALL	WR_LCD_DATA
	MOVLW	"."
	CALL	WR_LCD_DATA
	MOVFW	DP
	CALL	WR_LCD_DATA
	MOVLW	" "
	CALL	WR_LCD_DATA
	MOVLW	"C"
	CALL	WR_LCD_DATA
	RETURN		

CONVERT_VALUE	CLRF	DP	; .0 degree
	MOVLW	" "	; +C
	MOVWF	SIGN	
	BNC	POS_TEMP
NEG_TEMP	MOVLW	"-"	; -C 
	MOVWF	SIGN
	COMF	TEMP_RD,1	; 2's complement
	INCF	TEMP_RD,1	
POS_TEMP	CLRC
	RRF	TEMP_RD,1
	SKPC
	GOTO	LOOP1
	MOVLW	0x05	; .5 degree
	MOVWF	DP

LOOP1	MOVFW	TEMP_RD
	CLRF	H_BYTE
	MOVWF	L_BYTE
	CALL	B16TOBCD	; Convert number
	MOVLW	B'00001111'	; Mask bit
	ANDWF	R1,1
	MOVLW	" "
	MOVWF	R0
	TSTF	R1	; Test file R1=0 ?
	SKPNZ		; Skip if R1 <> 0
	GOTO	LOOP2
	MOVFW	R1
	CALL	TABLE
	MOVWF	R0 	
LOOP2 	SWAPF	R2,0
	ANDLW	B'00001111'	; Mask bit
	CALL	TABLE
	MOVWF	R1 
	MOVLW	B'00001111'	; Mask bit
	ANDWF	R2,0
	CALL	TABLE
	MOVWF	R2 
	MOVFW	DP
	CALL	TABLE
	MOVWF	DP
	RETURN
TABLE	ADDWF	PCL,1	; Number 0-9 
	DT              "0123456789"

RESET_PULSE	BCF	DQ_PIN  	; Logic 0
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY617U	; For RESET pulse
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = high impedance
	BCF	STATUS,RP0 	; Bank 0 
	RETURN

PRESENCE_PULSE  BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = high impedance 
	BCF	STATUS,RP0	; Bank 0 
	CALL	DELAY15U
	BTFSC	DQ_PIN 	; Skip if clear
	GOTO	$-1	; Wait for PRESENCE pulse
	BTFSS	DQ_PIN 	; Skip if DQ = idle
	GOTO	$-1
	RETURN
	
OW_WRITE	MOVWF	TEMP_WR
	MOVLW	.8
	MOVWF	COUNT8
WR_LOOP	RRF	TEMP_WR,1
	BC	SEND_1	; Branch if C = 1
SEND_0	BCF	DQ_PIN 
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY15U
	CALL	DELAY45U	
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = float
	BCF	STATUS,RP0	; Bank 0
	NOP
	GOTO	$+1
	DECFSZ	COUNT8,1
	GOTO	WR_LOOP
	RETURN
SEND_1	BCF	DQ_PIN 	
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY15U	
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = float
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY45U
	GOTO	$+1
	NOP
	DECFSZ	COUNT8,1
	GOTO	WR_LOOP
	RETURN		

OW_READ	MOVLW	.9
	MOVWF	COUNT8
READ_LOOP	BCF	DQ_PIN 
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	GOTO	$+1
	GOTO	$+1
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = input
	BCF	STATUS,RP0	; Bank 0
	BTFSC	DQ_PIN 	; Read data at DQ pin
	GOTO	READ_1
READ_0	CLRC
	RRF	TEMP_RD,1	
	CALL	DELAY45U
	DECFSZ	COUNT8,1
	GOTO	READ_LOOP
	RLF	TEMP_RD,1
	RETURN
READ_1	SETC
	RRF	TEMP_RD,1 	
	CALL	DELAY45U
	DECFSZ	COUNT8,1
	GOTO	READ_LOOP
	RLF	TEMP_RD,1
	RETURN
B16TOBCD  	CLRC                	; Clear the carry bit
	MOVLW   	.16
	MOVWF   	COUNT
	CLRF    	R0
	CLRF    	R1
	CLRF    	R2
LOOP16  	RLF     	L_BYTE,1
	RLF     	H_BYTE,1
	RLF     	R2,1
	RLF     	R1,1
	RLF     	R0,1
	DECFSZ  	COUNT,1
	GOTO    	ADJDEC
	RETLW   	0
ADJDEC  	MOVLW   	R2
	MOVWF   	FSR
	CALL    	ADJBCD
	MOVLW   	R1
	MOVWF   	FSR
	CALL  	ADJBCD
	MOVLW   	R0
	MOVWF   	FSR
	CALL    	ADJBCD
	GOTO    	LOOP16
ADJBCD  	MOVLW   	3
	ADDWF   	INDF,0
	MOVWF   	TEMP
	BTFSC   	TEMP,3          ; Test if result > 7
	MOVWF   	INDF 
	MOVLW   	0x30
	ADDWF   	INDF,0
	MOVWF   	TEMP
	BTFSC   	TEMP,7          ; Test if result > 7
	MOVWF   	INDF            ; Save as millisecond
	RETLW   	0

;+++++++++++++++++++++++++++++++++++++  LCD  +++++++++++++++++++++++++++++++++++++++
#DEFINE	LCD_DATA_BUS	PORTB	; RB7:RB4 = Data bus
#DEFINE	LOW_RS	BCF PORTB,3	; RB3 = RS pin
#DEFINE	HIGH_RS	BSF PORTB,3
#DEFINE	LOW_E	BCF PORTB,2	; RB2 = enable pin
#DEFINE	HIGH_E	BSF PORTB,2

    CONSTANT	FUNCTION_VALUE  = B'00101000'	; LCD command
    CONSTANT	ENTRY_VALUE     = B'00000110'	; LCD command
    CONSTANT	DISPLAY_VALUE   = B'00001100'	; LCD command

INIT_LCD	BSF	STATUS,RP0	; Bank 1
	MOVLW	B'00000011'	; RB7-RB2 as output
	MOVWF	TRISB		
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY5M	; Delay > 15 millisecond
	CALL	DELAY5M
	CALL	DELAY5M
	CALL	DELAY5M	
	CALL	FUNCTION_INIT	; 4 bit initialization
	CALL	FUNCTION_SET		
	CALL	ENTRY_MODE
	CALL	DISPLAY_CTRL
	CALL	CLEAR_SCREEN	; Clear display
	RETURN
	
FUNCTION_INIT	MOVLW	0x33		
	CALL	WR_LCD_CMD2 
	MOVLW	0x32	
	GOTO	WR_LCD_CMD	
FUNCTION_SET	MOVLW	FUNCTION_VALUE
	GOTO	WR_LCD_CMD 
ENTRY_MODE	MOVLW	ENTRY_VALUE
	GOTO	WR_LCD_CMD
DISPLAY_CTRL	MOVLW	DISPLAY_VALUE
	GOTO	WR_LCD_CMD

WR_LCD_DATA	MOVWF	BUFFER0	; Write high nibble
	CALL	MASK_BIT_DATA
	HIGH_E
	SWAPF	BUFFER0,0	; Write low nibble
	CALL	MASK_BIT_DATA
	HIGH_E
	CALL	DELAY45U	; Delay 40 microsecond
	RETURN
WR_LCD_CMD	MOVWF	BUFFER0	; Write high nibble
	CALL	MASK_BIT_CMD
	HIGH_E
	SWAPF	BUFFER0,0	; Write low nibble
	CALL	MASK_BIT_CMD
	CALL	DELAY45U	; Delay 40 microsecond
	HIGH_E
	RETURN
WR_LCD_CMD2	MOVWF	BUFFER0	; Write high nibble
	CALL	MASK_BIT_CMD
	CALL	DELAY5M
	HIGH_E
	SWAPF	BUFFER0,0	; Write low nibble
	CALL	MASK_BIT_CMD
	CALL	DELAY45U	; Delay > 100 microsecond
	CALL	DELAY45U
	CALL	DELAY45U	
	HIGH_E
	RETURN
MASK_BIT_CMD	MOVWF	BUFFER1
	LOW_RS
	BCF	BUFFER1,3	; For RS = 0
	BSF	BUFFER1,2	; For E  = 1
	MOVFW	BUFFER1
	MOVWF	LCD_DATA_BUS
	NOP
	LOW_E		; Enable pulse
	RETURN
MASK_BIT_DATA	MOVWF	BUFFER1
	HIGH_RS
	BSF	BUFFER1,3	; For RS  = 1
	BSF	BUFFER1,2	; For E   = 1
	MOVFW	BUFFER1
	MOVWF	LCD_DATA_BUS
	NOP
	LOW_E		; Enable pulse
	RETURN		
CLEAR_SCREEN	MOVLW	B'00000001'	; Clear LCD
	CALL	WR_LCD_CMD
	CALL	DELAY5M
	RETURN
RETURN_HOME	MOVLW	B'00000010' 	; Return home
	CALL	WR_LCD_CMD
	CALL	DELAY5M
	RETURN
DISPLAY_ON	MOVLW	B'00001100' 	; Turn-on display
	GOTO	WR_LCD_CMD
SET_POSITION	GOTO	WR_LCD_CMD	; Set DDRAM position

;++++++++++++++++++++++++++++++++ DELAY SUBROUTINES +++++++++++++++++++++++++++++++++
DELAY15U	MOVLW	.25	; Tdelay 15.05 microsecond
	MOVWF	XX
	DECFSZ	XX,1
	GOTO	$-1
	RETURN
DELAY45U	MOVLW	.74	; Delay ~45 microsecond
	MOVWF	XX
	DECFSZ	XX,1
	GOTO	$-1
	RETURN
DELAY617U	CLRF	XX	; Tdelay = 617.2 microsecond		
	MOVLW	.4
	MOVWF	YY
	DECFSZ	XX,1
	GOTO	$-1
	DECFSZ	YY,1
	GOTO	$-3
	RETURN
DELAY5M	CLRF	XX	; Delay ~5 millisecond		
	MOVLW	.33
	MOVWF	YY
	DECFSZ	XX,1
	GOTO	$-1
	DECFSZ	YY,1
	GOTO	$-3
	RETURN  

	END
